@@ -24,6 +24,11 @@
  * @file ucommon/unicode.h
  */
 
+/**
+ * An example of some unicode-utf8 transcoding.
+ * @example unicode.cpp
+ */
+
 #ifndef _UCOMMON_UNICODE_H_
 #define	_UCOMMON_UNICODE_H_
 
@@ -159,7 +164,7 @@ public:
 	 * @param file to get character from.
 	 * @return unicode character or EOF error.
 	 */
-	ucs4_t getchar(FILE *file);
+	ucs4_t getch(FILE *file);
 
 	/**
 	 * Push a unicode character to a file stream.
@@ -167,7 +172,7 @@ public:
 	 * @param file to push character to.
 	 * @return unicode character or EOF on error.
 	 */
-	ucs4_t putchar(ucs4_t character, FILE *file);
+	ucs4_t putch(ucs4_t character, FILE *file);
 };
 
 /**
